/*
 * ed_classes.C
 *
 *  Created on: 10 sty 2020
 *      Author: Daniel Wielanek
 *		E-mail: daniel.wielanek@gmail.com
 *		Warsaw University of Technology, Faculty of Physics
 */


#ifndef __CLING__

#include <TEveTrack.h>


#endif

class CompoundTrack;
class MyTrack: public TEveTrack{
public:
	MyTrack():TEveTrack(){};
	MyTrack(TParticle* t, Int_t label, TEveTrackPropagator* prop=0):TEveTrack(t,label,prop){};
	void HighlightThis(Bool_t state){TEvePointSet::HighlightElement(state);};
	virtual void HighlightElement(Bool_t state);
	virtual ~MyTrack(){};
	ClassDef(MyTrack,0)
};


class CompoundTrack: public TEveCompound{
	MyTrack *fRecoTrack;
	MyTrack *fMCTrack;
	Int_t fNPoints;
public:
	CompoundTrack(){
		fNPoints =0;
		fRecoTrack = new MyTrack();
		fMCTrack = new MyTrack();
		AddElement(fRecoTrack);
		AddElement(fMCTrack);
	}
	CompoundTrack(TParticle* t, Int_t label, TEveTrackPropagator* prop=0){
		fRecoTrack = new MyTrack(t,label,prop);
		fMCTrack = new MyTrack(t,label+1,prop);
		fNPoints =0;
		fRecoTrack->SetLineColor(kBlue);
		fMCTrack->SetLineColor(kRed);
		AddElement(fRecoTrack);
		AddElement(fMCTrack);
	}
	void HighlightElement(Bool_t state, TEveElement *el) {
		TEveCompound::HighlightElement(state);
		if(fRecoTrack!=el)
			fRecoTrack->HighlightThis(state);
		if(fMCTrack!=el)
			fMCTrack->HighlightThis(state);
	}


	void HighlightElement(Bool_t state){
		TEveCompound::HighlightElement(state);
		fRecoTrack->HighlightThis(state);
		fRecoTrack->HighlightThis(state);
	}
	 void  MakeTrack(Bool_t recurse=kTRUE){
		 fRecoTrack->MakeTrack(recurse);
		 fMCTrack->MakeTrack(recurse);
	 }

	 void SetFirstPoint(TVector3 mom, TVector3 pos) {
	 	fRecoTrack->SetPoint(fNPoints++, pos.X(),pos.Y(),pos.Z());
	 	fMCTrack->SetPoint(fNPoints-1, pos.X()+5,pos.Y()+5,pos.Z()+5);
	 	TEvePathMark path;
	 	Float_t point[3]={(Float_t)pos.X(),(Float_t)pos.Y(),(Float_t)pos.Z()};
	 	path.fV = point;
	 	path.fTime = 0;
	 	TEveVector Mom(mom.Px(),mom.Py(),mom.Pz());
	 	path.fP = Mom;
	 	fRecoTrack->AddPathMark(path);
	 	for(int i=0;i<3;i++)
	 		point[i] = point[i]+1;
	 	path.fV = point;
	 	fMCTrack->AddPathMark(path);
	 }

	 void AddPoint(TVector3 pos) {
	 	if(TMath::Abs(pos.X())>2000) return;
	 	if(TMath::Abs(pos.Y())>2000) return;
	 	if(TMath::Abs(pos.Z())>2000) return;
	 	if(TMath::IsNaN(pos.X())) return;
	 	fRecoTrack->SetPoint(fNPoints++, pos.X(),pos.Y(),pos.Z());
	 	fMCTrack->SetPoint(fNPoints-1, pos.X()+1,pos.Y()+1,pos.Z()+1);
	 	TEvePathMark path;
	 	Float_t point[3]={(Float_t)pos.X(),(Float_t)pos.Y(),(Float_t)pos.Z()};
	 	path.fV = point;
	 	path.fTime = 0;
	 	fRecoTrack->AddPathMark(path);
	 	for(int i=0;i<3;i++)
	 		point[i] = point[i]+1;
	 	path.fV = point;
	 	fMCTrack->AddPathMark(path);
	 }







	TEveTrack *GetTr1()const{return fRecoTrack;};
	TEveTrack *GetTr2()const{return fMCTrack;};
	virtual ~CompoundTrack(){};
	ClassDef(CompoundTrack,0)
};
/*


class CompoundTrack: public TEveTrack{
	TEveTrack *fRecoTrack;
	TEveTrack *fRecoTrack2;
public:
	CompoundTrack(){
		fRecoTrack = new TEveTrack();
		AddElement(fRecoTrack);
		fRecoTrack2 = new TEveTrack();
		AddElement(fRecoTrack2);
	}
	CompoundTrack(TEveRecTrack* t, TEveTrackPropagator* prop=0):TEveTrack(t,prop){
		fRecoTrack = new TEveTrack();
		AddElement(fRecoTrack);
		fRecoTrack2 = new TEveTrack();
		AddElement(fRecoTrack2);
	}
	 void  MakeTracks(Bool_t recurse=kTRUE){
		 fRecoTrack->MakeTrack(recurse);
	 }
	virtual ~CompoundTrack(){};
	ClassDef(CompoundTrack,1)
};
*/

class TrackList : public TEveTrackList{

public:
	TrackList(){fChildClass = CompoundTrack::Class();};
	TrackList(TString name, TEveTrackPropagator *prop=0):TEveTrackList(name,prop){
		fChildClass = CompoundTrack::Class();
	};
	   void  MakeTracks(Bool_t recurse=kTRUE){
		   for (List_i i=fChildren.begin(); i!=fChildren.end(); ++i)
		       {
			   CompoundTrack* track = dynamic_cast<CompoundTrack*>(*i);
		          if (track)
		          {
		             track->MakeTrack(recurse);

		          }
		          //if (recurse)
		             //FindMomentumLimits(*i, recurse);
		       }
	   }




	virtual ~TrackList(){};
	ClassDef(TrackList,0)
};

inline void MyTrack::HighlightElement(Bool_t state) {
	{
			TEveTrack::HighlightElement(state);
			for(List_i iter =BeginParents(); iter!=EndParents();++iter ){
				CompoundTrack *el  = dynamic_cast<CompoundTrack*>((*iter));
				if(el){
					el->HighlightElement(state,this);
				}
			}
		}
}
